@@ -1,25 +1,25 @@
-import numpy as np
 import os
-
-from emepy.mode import Mode
+import numpy as np
+import pickle
 
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torch.optim as optim
 
-import pickle
-
 from sklearn.preprocessing import PolynomialFeatures
-from emepy.fd import ModeSolver
-from emepy.tools import from_chunks
+import sklearn
 
+from emepy.mode import Mode, EigenMode
+from emepy.fd import ModeSolver
+from emepy.tools import from_chunks, get_epsfunc
+from emepy.materials import Si, SiO2
 
 FIELD_WIDTH = 128
 FIELD_SIZE = FIELD_WIDTH ** 2
 
 
-def getUpConvLayer(i_size, o_size, kernal, channels, first=False, last=False):
+def getUpConvLayer(i_size: int, o_size: int, kernal: int, channels: int, first: bool = False, last: bool = False):
     """Returns the right size for up convolutional sampling"""
 
     def out_size(in_size, kernal, stride, padding, output_padding):
@@ -43,35 +43,20 @@ def getUpConvLayer(i_size, o_size, kernal, channels, first=False, last=False):
 
     if first:
         return nn.ConvTranspose2d(
-            1,
-            channels,
-            stride=stride,
-            padding=padding,
-            kernel_size=kernal,
-            output_padding=output_padding,
+            1, channels, stride=stride, padding=padding, kernel_size=kernal, output_padding=output_padding
         )
 
     if last:
         return nn.ConvTranspose2d(
-            channels,
-            1,
-            stride=stride,
-            padding=padding,
-            kernel_size=kernal,
-            output_padding=output_padding,
+            channels, 1, stride=stride, padding=padding, kernel_size=kernal, output_padding=output_padding
         )
 
     return nn.ConvTranspose2d(
-        channels,
-        channels,
-        stride=stride,
-        padding=padding,
-        kernel_size=kernal,
-        output_padding=output_padding,
+        channels, channels, stride=stride, padding=padding, kernel_size=kernal, output_padding=output_padding
     )
 
 
-def getDownConvLayer(i_size, o_size, kernal, channels, first=False, last=False):
+def getDownConvLayer(i_size: int, o_size: int, kernal: int, channels: int, first: bool = False, last: bool = False):
     """Returns the right size for down convolutional sampling"""
 
     def out_size(in_size, kernal, stride, padding, dilation):
@@ -93,39 +78,18 @@ def getDownConvLayer(i_size, o_size, kernal, channels, first=False, last=False):
         raise Exception("Choose a different kernal size")
 
     if first:
-        return nn.Conv2d(
-            1,
-            channels,
-            stride=stride,
-            padding=padding,
-            kernel_size=kernal,
-            dilation=dilation,
-        )
+        return nn.Conv2d(1, channels, stride=stride, padding=padding, kernel_size=kernal, dilation=dilation)
 
     if last:
-        return nn.Conv2d(
-            channels,
-            1,
-            stride=stride,
-            padding=padding,
-            kernel_size=kernal,
-            dilation=dilation,
-        )
+        return nn.Conv2d(channels, 1, stride=stride, padding=padding, kernel_size=kernal, dilation=dilation)
 
-    return nn.Conv2d(
-        channels,
-        channels,
-        stride=stride,
-        padding=padding,
-        kernel_size=kernal,
-        dilation=dilation,
-    )
+    return nn.Conv2d(channels, channels, stride=stride, padding=padding, kernel_size=kernal, dilation=dilation)
 
 
 class Network(nn.Module):
     """The pytorch inherited class that defines and represents the physical neural network"""
 
-    def __init__(self, code_size, channels, component):
+    def __init__(self, code_size: int, channels: int, component: str) -> None:
         """Network constructor
 
         Parameters
@@ -141,30 +105,14 @@ class Network(nn.Module):
         self.channels = channels
 
         self.linear_up_1 = nn.Linear(code_size, int(FIELD_WIDTH / 20) ** 2)
-        self.linear_up_2 = nn.Linear(
-            int(FIELD_WIDTH / 20) ** 2, int(FIELD_WIDTH / 7) ** 2
-        )
-        self.linear_up_3 = nn.Linear(
-            int(FIELD_WIDTH / 7) ** 2, int(FIELD_WIDTH / 4) ** 2
-        )
-        self.linear_up_4 = nn.Linear(
-            int(FIELD_WIDTH / 4) ** 2, int(FIELD_WIDTH / 3) ** 2
-        )
-        self.conv_up_1 = getUpConvLayer(
-            int(FIELD_WIDTH / 3), int(FIELD_WIDTH / 2), 3, channels, first=True
-        )
-        self.conv_up_2 = getUpConvLayer(
-            int(FIELD_WIDTH / 2), int(5 * FIELD_WIDTH / 8), 5, channels
-        )
-        self.conv_up_3 = getUpConvLayer(
-            int(5 * FIELD_WIDTH / 8), int(6 * FIELD_WIDTH / 8), 7, channels
-        )
-        self.conv_up_4 = getUpConvLayer(
-            int(6 * FIELD_WIDTH / 8), int(7 * FIELD_WIDTH / 8), 7, channels
-        )
-        self.conv_up_5 = getUpConvLayer(
-            int(7 * FIELD_WIDTH / 8), int(FIELD_WIDTH), 9, channels, last=True
-        )
+        self.linear_up_2 = nn.Linear(int(FIELD_WIDTH / 20) ** 2, int(FIELD_WIDTH / 7) ** 2)
+        self.linear_up_3 = nn.Linear(int(FIELD_WIDTH / 7) ** 2, int(FIELD_WIDTH / 4) ** 2)
+        self.linear_up_4 = nn.Linear(int(FIELD_WIDTH / 4) ** 2, int(FIELD_WIDTH / 3) ** 2)
+        self.conv_up_1 = getUpConvLayer(int(FIELD_WIDTH / 3), int(FIELD_WIDTH / 2), 3, channels, first=True)
+        self.conv_up_2 = getUpConvLayer(int(FIELD_WIDTH / 2), int(5 * FIELD_WIDTH / 8), 5, channels)
+        self.conv_up_3 = getUpConvLayer(int(5 * FIELD_WIDTH / 8), int(6 * FIELD_WIDTH / 8), 7, channels)
+        self.conv_up_4 = getUpConvLayer(int(6 * FIELD_WIDTH / 8), int(7 * FIELD_WIDTH / 8), 7, channels)
+        self.conv_up_5 = getUpConvLayer(int(7 * FIELD_WIDTH / 8), int(FIELD_WIDTH), 9, channels, last=True)
         self.linear_up_5 = nn.Linear(FIELD_WIDTH ** 2, FIELD_WIDTH ** 2)
 
         self.relu = nn.ReLU()
@@ -181,7 +129,7 @@ class Network(nn.Module):
 
         self.component = component
 
-    def forward(self, field):
+    def forward(self, field) -> tuple:
         """Performs the network propagation
 
         Parameters
@@ -198,21 +146,11 @@ class Network(nn.Module):
         out = self.tanh(self.linear_up_1(field)).view(-1, 1, int(FIELD_WIDTH / 20) ** 2)
         out = self.tanh(self.linear_up_2(out)).view(-1, 1, int(FIELD_WIDTH / 7) ** 2)
         out = self.tanh(self.linear_up_3(out)).view(-1, 1, int(FIELD_WIDTH / 4) ** 2)
-        out = self.tanh(self.linear_up_4(out)).view(
-            -1, 1, int(FIELD_WIDTH / 3), int(FIELD_WIDTH / 3)
-        )
-        out = self.tanh(self.conv_up_1(out)).view(
-            -1, self.channels, int(FIELD_WIDTH / 2), int(FIELD_WIDTH / 2)
-        )
-        out = self.tanh(self.conv_up_2(out)).view(
-            -1, self.channels, int(5 * FIELD_WIDTH / 8), int(5 * FIELD_WIDTH / 8)
-        )
-        out = self.tanh(self.conv_up_3(out)).view(
-            -1, self.channels, int(6 * FIELD_WIDTH / 8), int(6 * FIELD_WIDTH / 8)
-        )
-        out = self.tanh(self.conv_up_4(out)).view(
-            -1, self.channels, int(7 * FIELD_WIDTH / 8), int(7 * FIELD_WIDTH / 8)
-        )
+        out = self.tanh(self.linear_up_4(out)).view(-1, 1, int(FIELD_WIDTH / 3), int(FIELD_WIDTH / 3))
+        out = self.tanh(self.conv_up_1(out)).view(-1, self.channels, int(FIELD_WIDTH / 2), int(FIELD_WIDTH / 2))
+        out = self.tanh(self.conv_up_2(out)).view(-1, self.channels, int(5 * FIELD_WIDTH / 8), int(5 * FIELD_WIDTH / 8))
+        out = self.tanh(self.conv_up_3(out)).view(-1, self.channels, int(6 * FIELD_WIDTH / 8), int(6 * FIELD_WIDTH / 8))
+        out = self.tanh(self.conv_up_4(out)).view(-1, self.channels, int(7 * FIELD_WIDTH / 8), int(7 * FIELD_WIDTH / 8))
         out = self.tanh(self.conv_up_5(out)).view(-1, 1, FIELD_WIDTH ** 2)
 
         out = self.linear_up_5(out).view(-1, FIELD_WIDTH, FIELD_WIDTH)
@@ -222,16 +160,63 @@ class Network(nn.Module):
         return out, field
 
 
+class ANN(object):
+    """Object that loads the neural network; Users are heavily encouraged to design their own networks and rewrite their own ANN to match their needs"""
+
+    def __init__(self,) -> None:
+        """Constructor for Mode Object"""
+        self.x = np.linspace(0, 2.5e-6, FIELD_WIDTH)
+        self.y = np.linspace(0, 2.5e-6, FIELD_WIDTH)
+
+        self.Hx_model = self.Hx_network()
+        self.Hy_model = self.Hy_network()
+        self.neff_model = self.neff_regression()
+
+    def neff_regression(self) -> "sklearn.linear_model._base.LinearRegression":
+        """Return the opened regression model for the effective index"""
+
+        with open(os.path.dirname(os.path.abspath(__file__)) + "/models/neff_pickle/model.pk", "rb") as f:
+            model = pickle.load(f)
+
+        return model
+
+    def Hx_network(self) -> Network:
+        """Return the opened network model for the Hx component"""
+
+        from_chunks(os.path.dirname(os.path.abspath(__file__)) + "/models/Hx_chunks/", "hx_temp.pt")
+        with open("hx_temp.pt", "rb") as f:
+            model = Network(3, 1, "Hx")
+
+            # original saved file with DataParallel
+            state_dict = torch.load(f)
+            model.load_state_dict(state_dict)
+
+            model.eval()
+
+        os.system("rm hx_temp.pt")
+        return model
+
+    def Hy_network(self) -> Network:
+        """Return the opened network model for the Hy component"""
+
+        from_chunks(os.path.dirname(os.path.abspath(__file__)) + "/models/Hy_chunks/", "hy_temp.pt")
+        with open("hy_temp.pt", "rb") as f:
+            model = Network(3, 1, "Hy")
+
+            # original saved file with DataParallel
+            state_dict = torch.load(f)
+            model.load_state_dict(state_dict)
+
+            model.eval()
+
+        os.system("rm hy_temp.pt")
+        return model
+
+
 class MSNeuralNetwork(ModeSolver):
     """ModeSolver object for the sample neural networks, parameterizes the cross section components. Currently designed only for single mode calculations in Silicon on SiO2"""
 
-    def __init__(
-        self,
-        ann,
-        wl,
-        width,
-        thickness,
-    ):
+    def __init__(self, ann: ANN, wl: float, width: float, thickness: float) -> None:
         """MSNeuralNetwork constructor
 
         Parameters
@@ -260,8 +245,11 @@ class MSNeuralNetwork(ModeSolver):
         self.after_y = self.y
         self.mesh = len(self.x) - 1
         self.PML = False
+        self.n = get_epsfunc(
+            self.width, self.thickness, 2.5e-6, 2.5e-6, Si(self.wl * 1e6), SiO2(self.wl * 1e6), compute=True
+        )(self.x, self.y)
 
-    def solve(self):
+    def solve(self) -> None:
         """Solves for the eigenmode using the neural networks"""
 
         self.mode = None
@@ -269,7 +257,7 @@ class MSNeuralNetwork(ModeSolver):
         Hx, Hy, neff = self.data(self.width, self.thickness, self.wl)
         self.mode = (Hx, Hy, neff)
 
-    def data(self, width, thickness, wl):
+    def data(self, width: float, thickness: float, wl: float) -> tuple:
         """Propagates the inputs into the neural networks and regression models and returns the outputs
 
         Parameters
@@ -293,7 +281,9 @@ class MSNeuralNetwork(ModeSolver):
 
         return Hx, Hy, neff
 
-    def neff_regression(self, width, thickness, wl, model):
+    def neff_regression(
+        self, width: float, thickness: float, wl: float, model: "sklearn.linear_model._base.LinearRegression"
+    ) -> float:
         """Calculates the effective index using a regression model
 
         Parameters
@@ -319,7 +309,7 @@ class MSNeuralNetwork(ModeSolver):
 
         return neff[0]
 
-    def Hx_network(self, width, thickness, wl, model):
+    def Hx_network(self, width: float, thickness: float, wl: float, model: Network) -> "np.ndarray":
         """Calculates the Hx component using a network model
 
         Parameters
@@ -347,7 +337,7 @@ class MSNeuralNetwork(ModeSolver):
 
         return output
 
-    def Hy_network(self, width, thickness, wl, model):
+    def Hy_network(self, width: float, thickness: float, wl: float, model: Network) -> "np.ndarray":
         """Calculates the Hy component using a network model
 
         Parameters
@@ -375,13 +365,29 @@ class MSNeuralNetwork(ModeSolver):
 
         return output
 
-    def clear(self):
+    def clear(self) -> None:
         """Clears the mode in the object"""
         self.mode = None
 
-    def get_mode(self, mode_num=0):
-        """Returns the solved eigenmode"""
+    def get_mode(self, mode_num: int = 0) -> EigenMode:
+        """Returns the solved eigenmode
+
+        Parameters
+        ----------
+        mode_num : int
+            mode index to return mode of
+
+        Returns
+        -------
+        EigenMode
+            the EigenMode corresponding to the provdided mode index
+        """
         Hx, Hy, neff = self.mode
+
+        epsfunc_before = get_epsfunc(
+            self.width, self.thickness, 2.5e-6, 2.5e-6, Si(self.wl * 1e6), SiO2(self.wl * 1e6), compute=True
+        )
+        epsfunc_after = get_epsfunc(self.width, self.thickness, 2.5e-6, 2.5e-6, Si(self.wl * 1e6), SiO2(self.wl * 1e6))
         m = Mode(
             self.x,
             self.y,
@@ -393,74 +399,9 @@ class MSNeuralNetwork(ModeSolver):
             None,
             None,
             None,
-            width=self.width,
-            thickness=self.thickness,
+            np.sqrt(epsfunc_after(self.x, self.y)),
         )
-        m.compute_other_fields()
+        m.compute_other_fields(epsfunc_before, epsfunc_after)
         m.normalize()
 
         return m
-
-
-class ANN(object):
-    """Object that loads the neural network; Users are heavily encouraged to design their own networks and rewrite their own ANN to match their needs"""
-
-    def __init__(
-        self,
-    ):
-        """Constructor for Mode Object"""
-        self.x = np.linspace(0, 2.5e-6, FIELD_WIDTH)
-        self.y = np.linspace(0, 2.5e-6, FIELD_WIDTH)
-
-        self.Hx_model = self.Hx_network()
-        self.Hy_model = self.Hy_network()
-        self.neff_model = self.neff_regression()
-
-    def neff_regression(self):
-        """Return the opened regression model for the effective index"""
-
-        with open(
-            os.path.dirname(os.path.abspath(__file__)) + "/models/neff_pickle/model.pk",
-            "rb",
-        ) as f:
-            model = pickle.load(f)
-
-        return model
-
-    def Hx_network(self):
-        """Return the opened network model for the Hx component"""
-
-        from_chunks(
-            os.path.dirname(os.path.abspath(__file__)) + "/models/Hx_chunks/",
-            "hx_temp.pt",
-        )
-        with open("hx_temp.pt", "rb") as f:
-            model = Network(3, 1, "Hx")
-
-            # original saved file with DataParallel
-            state_dict = torch.load(f)
-            model.load_state_dict(state_dict)
-
-            model.eval()
-
-        os.system("rm hx_temp.pt")
-        return model
-
-    def Hy_network(self):
-        """Return the opened network model for the Hy component"""
-
-        from_chunks(
-            os.path.dirname(os.path.abspath(__file__)) + "/models/Hy_chunks/",
-            "hy_temp.pt",
-        )
-        with open("hy_temp.pt", "rb") as f:
-            model = Network(3, 1, "Hy")
-
-            # original saved file with DataParallel
-            state_dict = torch.load(f)
-            model.load_state_dict(state_dict)
-
-            model.eval()
-
-        os.system("rm hy_temp.pt")
-        return model
