@@ -1,12 +1,12 @@
 from emepy.fd import MSEMpy
 import importlib
 import EMpy_gpu
-if not (importlib.util.find_spec("pycuda") is None):
+
+if importlib.util.find_spec("pycuda") is not None:
     import pycuda.driver as cuda
     import pycuda.autoinit
 
 
-
 class MSEMpyGPU(MSEMpy):
     """ Exactly the same as MSEMpy except allocates space and solves eigenvectors and values on the GPU
     """
@@ -17,5 +17,3 @@ class MSEMpyGPU(MSEMpy):
             self.num_modes, self.accuracy
         )
         return self
-
-    
\ No newline at end of file
