@@ -1,6 +1,8 @@
 import numpy as np
+import numpy
 from matplotlib import pyplot as plt
-from copy import copy
+import matplotlib
+from copy import deepcopy
 from scipy.interpolate import griddata
 
 
@@ -9,43 +11,45 @@ class Monitor(object):
 
     def __init__(
         self,
-        axes="xz",
-        dimensions=(1, 1),
-        lengths=[],
-        components=["E"],
-        z_range=None,
-        grid_x=None,
-        grid_y=None,
-        grid_z=None,
-        location=None,
-    ):
-        """Monitor class constructor
+        axes: str = "xz",
+        dimensions: tuple = (1, 1),
+        components: list = ["E"],
+        z_range: tuple = None,
+        grid_x: np.array = None,
+        grid_y: np.array = None,
+        grid_z: np.array = None,
+        location: float = None,
+        sources: list = [],
+    ) -> None:
+        """Monitor class constructor0
 
         Parameters
         ----------
         axes : string
-            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'yz', 'xyz', 'x', 'y', 'z'. Note, propagation is always in z.
+            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'yz', 'xyz', 'x', 'y', 'z'. Note, propagation is always in z. (default: "xy")
         dimensions : tuple
-            the spacial dimensions of the resulting field
-        lengths : list
-            list of the remaining points in z at which to calculate the fields
+            the spacial dimensions of the resulting field (default: (1,1))
         components : list
-            list of the field components to store from ('E','H','Ex','Ey','Ez','Hx','Hy','Hz)
+            list of the field components to store from ('E','H','Ex','Ey','Ez','Hx','Hy','Hz) (default: ["E"])
         z_range : tuple
-            tuple or list of the form (start, end) representing the range of the z values to extract
-        grid_x : numpy array
+            tuple or list of the form (start, end) representing the range of the z values to extract (default: None)
+        grid_x : numpy array (default: None)
             1d x grid
-        grid_y : numpy array
+        grid_y : numpy array (default: None)
             1d y grid
-        grid_z : numpy array
+        grid_z : numpy array (default: None)
             1d z grid
+        location : float
+            the location in z if the monitor represents "xy" axes (default: None)
+        sources : list[Source]
+            sources to use for the monitor (default:[])
         """
 
         # Ensure z range is in proper format
         if not (axes in ["xy", "yx"]):
             try:
                 if z_range is None:
-                    self.start, self.end = [lengths[0][0], lengths[0][-1]]
+                    self.start, self.end = [grid_z[0], grid_z[-1]]
                 else:
                     self.start, self.end = z_range
             except Exception as e:
@@ -55,6 +59,7 @@ class Monitor(object):
                     "and end are floats such as (0, 1e-6) for a 1 µm range"
                 ) from e
 
+        # Check axes
         if axes == "xz" or axes == "zx":
             self.axes = "xz"
         elif axes == "yz" or axes == "zy":
@@ -69,55 +74,65 @@ class Monitor(object):
                 " the following implemented monitor types: ['xy','yz','xz','xyz']"
             )
 
+        # Set parameters globally
         self.dimensions = dimensions
         self.field = np.zeros(dimensions).astype(complex)
-        self.lengths = copy(lengths)
-        self.remaining_lengths = copy(lengths)
-
-        self.cur_prop_index = [0 for i in range(len(components))]
-        self.cur_length = [0 for i in range(len(components))]
+        self.lengths = grid_z.tolist()
+        self.sources = sources
+        self.left_source = False
+        self.right_source = False
         self.components = components
         self.layers = {}
         self.grid_x = grid_x
         self.grid_y = grid_y
         self.grid_z = grid_z
+        self.location = location
 
-    def __getitem__(self, subscript):
-        return self.field[subscript]
+    def reset_monitor(self) -> None:
+        """Resets the fields in the monitor"""
+        self.field *= 0
+        # self.sources = []
+        # self.left_source = False
+        # self.right_source = False
 
-    def __setitem__(self, subscript, item):
+    def get_z_list(self, start: float, end: float) -> list:
+        """Finds all the points in z between start and end
 
-        # Only set item if it's in the valid z_range
-        if self.axes in ["xy", "yx"]:
-            self.field[subscript] = item
-        elif (self.lengths[0][subscript[-1]] >= self.start) and (self.lengths[0][subscript[-1]] <= self.end):
-            difference_start = lambda list_value: abs(list_value - self.start)
-            s = self.lengths[0].index(min(self.lengths[0], key=difference_start))
-            subscript = tuple(list(subscript[:-1]) + [subscript[-1] - s])
-            self.field[subscript] = item
+        Parameters
+        ----------
+        start : float
+            starting point in z
+        end : float
+            ending point in z
 
-        if len(self.remaining_lengths[int(subscript[0])]) > 1:
-            self.remaining_lengths[int(subscript[0])] = self.remaining_lengths[int(subscript[0])][1:]
-            self.cur_prop_index[int(subscript[0])] += 1
-        else:
-            self.remaining_lengths[int(subscript[0])] = []
-            self.cur_prop_index[int(subscript[0])] += 1
+        Returns
+        -------
+        list[tuples]
+            A list of tuples that take the format (i, l) where i is the index of the z point and l is the z point for all z points in the range
+        """
+        return [(i, l) for i, l in enumerate(self.lengths) if start <= l <= end]
 
-    def __delitem__(self, subscript):
-        del self.field[subscript]
-
-    def normalize(self):
+    def normalize(self) -> None:
+        """Normalizes the entire field to 1"""
         self.field[:-1] /= 1  # np.max(np.abs(self.field[:-1, :, 0]))
 
-    def get_array(self, component="Hy", axes=None, location=None, z_range=None, grid_x=None, grid_y=None):
+    def get_array(
+        self,
+        component: str = "Hy",
+        axes: str = None,
+        location: float = None,
+        z_range: tuple = None,
+        grid_x: np.array = None,
+        grid_y: np.array = None,
+    ) -> "numpy.ndarray":
         """Creates a matplotlib axis displaying the provides field component
 
         Parameters
         ----------
-        axes : string
-            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'yz', 'xyz', 'x', 'y', 'z'. Note, propagation is always in z.
-        component : string
+        component : str
             field component from "['Ex','Ey','Ez','Hx','Hy','Hz','E','H']"
+        axes : str
+            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'yz', 'xyz', 'x', 'y', 'z'. Note, propagation is always in z.
         location : float
             if taken from 3D fields, users can specify where to take their 2D slice. If axes is 'xz', location refers to the location in y and 'yz' refers to a location in x and 'xy' refers to a location in z
         z_range : tuple
@@ -131,7 +146,6 @@ class Monitor(object):
         -------
         numpy array
             the requested field
-
         """
 
         # Default axes is created upon class creation
@@ -167,14 +181,14 @@ class Monitor(object):
             def difference_end(list_value):
                 return abs(list_value - end)
 
-            s = self.lengths[0].index(min(self.lengths[0], key=difference_start))
-            e = self.lengths[0].index(min(self.lengths[0], key=difference_end)) + 1
-            default_grid_z = self.lengths[0][s:e]
+            s = self.lengths.index(min(self.lengths, key=difference_start))
+            e = self.lengths.index(min(self.lengths, key=difference_end)) + 1
+            default_grid_z = self.lengths[s:e]
 
             def m(list_value):
                 return abs(list_value - self.grid_z[0])
 
-            m = self.lengths[0].index(min(self.lengths[0], key=m))
+            m = self.lengths.index(min(self.lengths, key=m))
             s -= m
             e -= m
 
@@ -310,22 +324,54 @@ class Monitor(object):
 
         return grid_field
 
-    def visualize(self, ax=None, component="Hy", axes=None, location=0, z_range=None):
+    def get_source_visual(self, shape) -> "numpy.ndarray":
+        """Returns a mask with lines indicating where a source is"""
+        array = np.zeros(shape)
+        l = self.dimensions[-1] // 200
+        if self.right_source:
+            array[:, -l:] = 1
+        if self.left_source:
+            array[:, :l] = 1
+        for source in self.sources:
+            difference_start = lambda list_value: abs(list_value - source.z)
+            i = self.lengths.index(min(self.lengths, key=difference_start))
+            array[:, i - l // 2 : i + l // 2] = 1
+
+        return array
+
+    def visualize(
+        self,
+        ax: matplotlib.image.AxesImage = None,
+        component: str = "Hy",
+        axes: str = None,
+        location: float = 0,
+        z_range: tuple = None,
+        show_geometry: bool = True,
+        show_sources: bool = True,
+    ) -> "matplotlib.image.AxesImage":
         """Creates a matplotlib axis displaying the provides field component
 
         Parameters
         ----------
-        axes : string
-            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'yz', 'xyz', 'x', 'y', 'z'. Note, propagation is always in z.
         ax : matplotlib axis
             the axis object created when calling plt.figure() or plt.subplots(), if None (default) then the plt interface will be used
         component : string
             field component from "['Ex','Ey','Ez','Hx','Hy','Hz','E','H']"
+        axes : string
+            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'yz', 'xyz', 'x', 'y', 'z'. Note, propagation is always in z.
         location : float
             if taken from 3D fields, users can specify where to take their 2D slice. If axes is 'xz', location refers to the location in y and 'yz' refers to a location in x and 'xy' refers to a location in z.
         z_range : tuple
             tuple or list of the form (start, end) representing the range of the z values to extract
+        show_geometry : bool
+            if true, will display the geometry faintly under the field profiles (default: True)
+        show_sources : bool
+            if true, will display a red line indicating where a source is (default: True)
 
+        Returns
+        -------
+        matplotlib.image.AxesImage
+            the image used to plot the index profile
         """
 
         # Only 2D fields can be generated
@@ -346,6 +392,7 @@ class Monitor(object):
         else:
             raise Exception("Incorrect axes format")
 
+        yn, zn, n = self.get_array(component="n", axes=axes, location=location, z_range=z_range)
         y, z, field = self.get_array(component=component, axes=axes, location=location, z_range=z_range)
 
         # Color map lookup table
@@ -364,19 +411,43 @@ class Monitor(object):
         # Create plots
         if axes in ["xz", "zx", "yz", "zy"]:
             if ax:
+                if show_geometry:
+                    ax.imshow(
+                        np.real(n),
+                        extent=[np.real(zn[0]), np.real(zn[-1]), np.real(yn[0]), np.real(yn[-1])],
+                        cmap=cmap_lookup["n"],
+                    )
+                vmin, vmax = (np.real(np.min(field)), np.real(np.max(field)))
+                if show_sources:
+                    srcs = np.real(self.get_source_visual(field.shape))
+                    field = np.where(srcs, -max(np.abs(vmax), np.abs(vmin)) * 1000, field)
                 im = ax.imshow(
                     np.real(field),
                     extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
                     cmap=cmap_lookup[component],
+                    alpha=1 if not show_geometry else 0.85,
                 )
                 ax.set_xlabel(np.real(axes[1]))
                 ax.set_ylabel(np.real(axes[0]))
                 ax.set_title(component)
             else:
+                if show_geometry:
+                    plt.imshow(
+                        np.real(n),
+                        extent=[np.real(zn[0]), np.real(zn[-1]), np.real(yn[0]), np.real(yn[-1])],
+                        cmap=cmap_lookup["n"],
+                    )
+                vmin, vmax = (np.real(np.min(field)), np.real(np.max(field)))
+                if show_sources:
+                    srcs = np.real(self.get_source_visual(field.shape))
+                    field = np.where(srcs, -max(np.abs(vmax), np.abs(vmin)) * 1000, field)
                 im = plt.imshow(
                     np.real(field),
                     extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
                     cmap=cmap_lookup[component],
+                    alpha=1 if not show_geometry else 0.85,
+                    vmin=vmin,
+                    vmax=vmax,
                 )
                 plt.xlabel(np.real(axes[1]))
                 plt.ylabel(np.real(axes[0]))
@@ -402,3 +473,12 @@ class Monitor(object):
                 plt.title(component)
 
         return im
+
+    def __getitem__(self, subscript):
+        return self.field[subscript]
+
+    def __setitem__(self, subscript, item):
+        self.field[subscript] = item
+
+    def __delitem__(self, subscript):
+        del self.field[subscript]
